@@ -597,6 +597,11 @@ Antialias Context::get_antialias() const
   return result;
 }
 
+bool Context::has_current_point() const
+{
+  return cairo_has_current_point(m_cobject);
+}
+
 void Context::get_current_point(double& x, double& y) const
 {
   cairo_get_current_point(m_cobject, &x, &y);
@@ -733,6 +738,13 @@ Path* Context::copy_path() const
   return new Path(cresult, true /* take ownership */); //The caller must delete it.
 }
 
+void Context::get_path_extents(double& x1, double& y1, double& x2, double& y2) const
+{
+  cairo_path_extents(m_cobject, &x1, &y1, &x2, &y2);
+  check_object_status_and_throw_exception(*this);
+}
+
+
 Path* Context::copy_path_flat() const
 {
   cairo_path_t* cresult = cairo_copy_path_flat(const_cast<cairo_t*>(m_cobject));
