@@ -19,6 +19,8 @@
 #ifndef __CAIROMM_CONTEXT_H
 #define __CAIROMM_CONTEXT_H
 
+#include <vector>
+#include <utility>
 #include <cairomm/surface.h>
 #include <cairomm/fontface.h>
 #include <cairomm/pattern.h>
@@ -35,6 +37,7 @@ typedef cairo_glyph_t Glyph; //A simple struct.
 typedef cairo_font_extents_t FontExtents; //A simple struct.
 typedef cairo_text_extents_t TextExtents; //A simple struct.
 typedef cairo_matrix_t Matrix; //A simple struct. //TODO: Derive and add operator[] and operator. matrix multiplication?
+typedef cairo_rectangle_t Rectangle;
 
 /** Context is the main class used to draw in cairomm. 
  * In the simplest case, create a Context with its target Surface, set its
@@ -42,9 +45,7 @@ typedef cairo_matrix_t Matrix; //A simple struct. //TODO: Derive and add operato
  * move_to() and line_to(), and then draw the shapes to the Surface using
  * methods such as stroke() or fill().
  *
- * Context is a reference-counted object. The copy constructor creates a second
- * reference to the object, instead of creating an independent copy of the
- * object.
+ * Context is a reference-counted object that should be used via Cairo::RefPtr.
  */
 class Context
 {
@@ -234,6 +235,11 @@ public:
    */
   void set_line_join(LineJoin line_join);
 
+  /** 
+   * Alternate version of set_dash().  You'll probably want to use the one that
+   * takes a std::vector argument instead.
+   */
+  void set_dash(std::valarray<double>& dashes, double offset);
   /** Sets the dash pattern to be used by stroke(). A dash pattern is specified
    * by dashes, an array of positive values. Each value provides the user-space
    * length of altenate "on" and "off" portions of the stroke. The offset
@@ -249,9 +255,9 @@ public:
    * @param dashes	an array specifying alternate lengths of on and off portions
    * @param offset	an offset into the dash pattern at which the stroke should start
    *
-   * \exception
+   * @exception
    */
-  void set_dash(std::valarray<double>& dashes, double offset);
+  void set_dash(std::vector<double>& dashes, double offset);
 
   /** This function disables a dash pattern that was set with set_dash()
    */
@@ -351,10 +357,22 @@ public:
    */
   void device_to_user_distance(double& dx, double& dy);
 
-  // FIXME: why is this named clear_path instead of new_path?
   /** Clears the current path. After this call there will be no current point.
    */
-  void clear_path();
+  void begin_new_path();
+
+  /** Begin a new subpath. Note that the existing path is not affected. After
+   * this call there will be no current point.
+   *
+   * In many cases, this call is not needed since new subpaths are frequently
+   * started with move_to().
+   *
+   * A call to begin_new_sub_path() is particularly useful when beginning a new
+   * subpath with one of the arc() calls. This makes things easier as it is no
+   * longer necessary to manually compute the arc's initial coordinates for a
+   * call to move_to().
+   */
+  void begin_new_sub_path();
 
   /** If the current subpath is not empty, begin a new subpath. After this call
    * the current point will be (x, y).
@@ -554,7 +572,7 @@ public:
    *
    * @param pattern a Pattern
    */
-  void mask(const RefPtr<Pattern>& pattern);
+  void mask(const RefPtr<const Pattern>& pattern);
 
   /** A drawing operator that paints the current source using the alpha channel
    * of surface as a mask. (Opaque areas of surface are painted with the
@@ -564,17 +582,17 @@ public:
    * @param surface_x	X coordinate at which to place the origin of surface
    * @param surface_y	Y coordinate at which to place the origin of surface
    */
-  void mask(const RefPtr<Surface>& surface, double surface_x, double surface_y);
+  void mask(const RefPtr<const Surface>& surface, double surface_x, double surface_y);
 
   /** A drawing operator that strokes the current Path according to the current
    * line width, line join, line cap, and dash settings. After stroke(),
    * the current Path will be cleared from the cairo Context.
    *
-   * \sa set_line_width()
-   * \sa set_line_join()
-   * \sa set_line_cap()
-   * \sa set_dash()
-   * \sa stroke_preserve().
+   * @sa set_line_width()
+   * @sa set_line_join()
+   * @sa set_line_cap()
+   * @sa set_dash()
+   * @sa stroke_preserve().
    */
   void stroke();
 
@@ -582,11 +600,11 @@ public:
    * line width, line join, line cap, and dash settings. Unlike stroke(),
    * stroke_preserve() preserves the Path within the cairo Context.
    *
-   * \sa set_line_width()
-   * \sa set_line_join()
-   * \sa set_line_cap()
-   * \sa set_dash()
-   * \sa stroke_preserve().
+   * @sa set_line_width()
+   * @sa set_line_join()
+   * @sa set_line_cap()
+   * @sa set_dash()
+   * @sa stroke_preserve().
    */
   void stroke_preserve();
 
@@ -594,8 +612,8 @@ public:
    * fill rule, (each sub-path is implicitly closed before being filled). After
    * fill(), the current path will be cleared from the cairo context. 
    *
-   * \sa set_fill_rule() 
-   * \sa fill_preserve()
+   * @sa set_fill_rule() 
+   * @sa fill_preserve()
    */
   void fill();
 
@@ -604,8 +622,8 @@ public:
    * Unlike fill(), fill_preserve() preserves the path within the
    * cairo Context.
    *
-   * \sa set_fill_rule() 
-   * \sa fill().
+   * @sa set_fill_rule() 
+   * @sa fill().
    */
   void fill_preserve();
   void copy_page();
@@ -643,7 +661,7 @@ public:
    * save()/restore() pair. The only other means of increasing the size of the
    * clip region is reset_clip().
    *
-   * \sa set_fill_rule()
+   * @sa set_fill_rule()
    */
   void clip();
 
@@ -654,10 +672,34 @@ public:
    * Unlike clip(), cairo_clip_preserve preserves the path within the cairo
    * Context. 
    *
-   * \sa clip()
-   * \sa set_fill_rule()
+   * @sa clip()
+   * @sa set_fill_rule()
    */
   void clip_preserve();
+
+  /**
+   * Computes a bounding box in user coordinates covering the area inside the
+   * current clip.
+   *
+   * @param x1 left of the resulting extents
+   * @param y1 top of the resulting extents
+   * @param x2 right of the resulting extents
+   * @param y2 bottom of the resulting extents
+   *
+   * @since 1.4
+   **/
+  void get_clip_extents(double& x1, double& y1, double& x2, double& y2) const;
+
+  /**
+   * Returns the current clip region as a list of rectangles in user coordinates.
+   *
+   * This function will throw an exception if the clip region cannot be
+   * represented as a list of user-space rectangles.
+   *
+   * @Since 1.4
+   **/
+   void copy_clip_rectangle_list(std::vector<Rectangle>& rectangles) const;
+
   void select_font_face(const std::string& family, FontSlant slant, FontWeight weight);
   void set_font_size(double size);
   void set_font_matrix(const Matrix& matrix);
@@ -704,9 +746,15 @@ public:
    *
    * @param x	return value for X coordinate of the current point
    * @param y	return value for Y coordinate of the current point
+   *
+   * @see has_current_point
    */
   void get_current_point (double& x, double& y) const;
 
+  /** Checks if there is a current point defined
+   */
+  bool has_current_point() const;
+
   /** Gets the current fill rule, as set by set_fill_rule().
    */
   FillRule get_fill_rule() const;
@@ -727,6 +775,17 @@ public:
    */
   double get_miter_limit() const;
 
+  /**
+   * Gets the current dash array and offset.
+   *
+   * @param dashes return value for the dash array
+   * @param offset return value for the current dash offset
+   *
+   * Since: 1.4
+   **/
+  void get_dash(std::vector<double>& dashes, double& offset) const;
+
+
   /** Stores the current transformation matrix (CTM) into matrix.
    *
    * @param matrix	return value for the matrix
@@ -744,9 +803,62 @@ public:
    * @exception
    */
   RefPtr<const Surface> get_target() const;
-  
+
   //TODO: Copy or reference-count a Path somethow instead of asking the caller to delete it?
+  /** Creates a copy of the current path and returns it to the user.
+   *
+   * @todo See cairo_path_data_t for hints on how to iterate over the returned
+   * data structure.
+   *
+   * @note The caller owns the Path object returned from this function.  The
+   * Path object must be freed when you are finished with it.
+   */
   Path* copy_path() const;
+
+/**
+ * Computes a bounding box in user-space coordinates covering the points on
+ * the current path. If the current path is empty, returns an empty rectangle
+ * ((0,0), (0,0)). Stroke parameters, fill rule, surface dimensions and
+ * clipping are not taken into account.
+ *
+ * Contrast with fill_extents() and stroke_extents() which return the extents
+ * of only the area that would be "inked" by the corresponding drawing
+ * operations.
+ *
+ * The result of path_extents() is defined as equivalent to the limit of
+ * stroke_extents() with Cairo::LINE_CAP_ROUND as the line width approaches
+ * 0.0, (but never reaching the empty-rectangle returned by stroke_extents()
+ * for a line width of 0.0).
+ *
+ * Specifically, this means that zero-area sub-paths such as
+ * move_to();line_to() segments, (even degenerate cases where the coordinates
+ * to both calls are identical), will be considered as contributing to the
+ * extents. However, a lone move_to() will not contribute to the results of
+ * path_extents().
+ *
+ * @param x1 left of the resulting extents
+ * @param y1 top of the resulting extents
+ * @param x2 right of the resulting extents
+ * @param y2 bottom of the resulting extents
+ *
+ * @since 1.6
+ **/
+  void get_path_extents(double& x1, double& y1, double& x2, double& y2) const;
+
+  /** Gets a flattened copy of the current path and returns it to the user
+   *
+   * @todo See cairo_path_data_t for hints on how to iterate over the returned
+   * data structure.
+   *
+   * This function is like copy_path() except that any curves in the path will
+   * be approximated with piecewise-linear approximations, (accurate to within
+   * the current tolerance value). That is, the result is guaranteed to not have
+   * any elements of type CAIRO_PATH_CURVE_TO which will instead be
+   * replaced by a series of CAIRO_PATH_LINE_TO elements. 
+   *
+   * @note The caller owns the Path object returned from this function.  The
+   * Path object must be freed when you are finished with it.
+   */
   Path* copy_path_flat() const;
 
   /** Append the path onto the current path. The path may be either the return
@@ -757,6 +869,113 @@ public:
    */
   void append_path(const Path& path);
 
+  /** Temporarily redirects drawing to an intermediate surface known as a group.
+   * The redirection lasts until the group is completed by a call to pop_group()
+   * or pop_group_to_source(). These calls provide the result of any drawing to
+   * the group as a pattern, (either as an explicit object, or set as the source
+   * pattern).
+   *
+   * This group functionality can be convenient for performing intermediate
+   * compositing. One common use of a group is to render objects as opaque
+   * within the group, (so that they occlude each other), and then blend the
+   * result with translucence onto the destination.
+   *
+   * Groups can be nested arbitrarily deep by making balanced calls to
+   * push_group()/pop_group(). Each call pushes/pops the new target group
+   * onto/from a stack.
+   *
+   * The push_group() function calls save() so that any changes to the graphics
+   * state will not be visible outside the group, (the pop_group functions call
+   * restore()).
+   *
+   * By default the intermediate group will have a content type of
+   * CONTENT_COLOR_ALPHA. Other content types can be chosen for the group by
+   * using push_group_with_content() instead.
+   *
+   * As an example, here is how one might fill and stroke a path with
+   * translucence, but without any portion of the fill being visible under the
+   * stroke:
+   *
+   * @code
+   * cr->push_group();
+   * cr->set_source(fill_pattern);
+   * cr->fill_preserve();
+   * cr->set_source(stroke_pattern);
+   * cr->stroke();
+   * cr->pop_group_to_source();
+   * cr->paint_with_alpha(alpha);
+   * @endcode
+   */
+  void push_group();
+
+  /**
+   * Temporarily redirects drawing to an intermediate surface known as a
+   * group. The redirection lasts until the group is completed by a call
+   * to pop_group() or pop_group_to_source(). These calls provide the result of
+   * any drawing to the group as a pattern, (either as an explicit object, or set
+   * as the source pattern).
+   *
+   * The group will have a content type of @content. The ability to control this
+   * content type is the only distinction between this function and push_group()
+   * which you should see for a more detailed description of group rendering.
+   *
+   * @param content: indicates the type of group that will be created
+   */
+  void push_group_with_content(Content content);
+
+  /**
+   * Terminates the redirection begun by a call to push_group() or
+   * push_group_with_content() and returns a new pattern containing the results
+   * of all drawing operations performed to the group.
+   *
+   * The pop_group() function calls restore(), (balancing a call to save() by
+   * the push_group function), so that any changes to the graphics state will
+   * not be visible outside the group.
+   *
+   * @return a (surface) pattern containing the results of all drawing
+   * operations performed to the group.
+   **/
+  RefPtr<Pattern> pop_group();
+
+  /**
+   * Terminates the redirection begun by a call to push_group() or
+   * push_group_with_content() and installs the resulting pattern as the source
+   * pattern in the given cairo Context.
+   *
+   * The behavior of this function is equivalent to the sequence of operations:
+   *
+   * @code
+   * RefPtr<Pattern> group = cr->pop_group();
+   * cr->set_source(group);
+   * @endcode
+   *
+   * but is more convenient as their is no need for a variable to store
+   * the short-lived pointer to the pattern.
+   *
+   * The pop_group() function calls restore(), (balancing a call to save() by
+   * the push_group function), so that any changes to the graphics state will
+   * not be visible outside the group.
+   **/
+  void pop_group_to_source();
+
+  /**
+   * Gets the target surface for the current group as started by the most recent
+   * call to push_group() or push_group_with_content().
+   *
+   * This function will return NULL if called "outside" of any group rendering
+   * blocks, (that is, after the last balancing call to pop_group() or
+   * pop_group_to_source()).
+   *
+   * @exception
+   *
+   **/
+  RefPtr<Surface> get_group_target();
+
+  /**
+   * Same as the non-const version but returns a reference to a const Surface
+   */
+  RefPtr<const Surface> get_group_target() const;
+
   /** The base cairo C type that is wrapped by Cairo::Context
    */
   typedef cairo_t cobject;
