@@ -28,6 +28,7 @@
 
 //See xlib_surface.h for XlibSurface.
 //See win32_surface.h for Win32Surface.
+//See quartz_surface.h for QuartzSurface (Mac OS X).
 
 #ifdef CAIRO_HAS_PDF_SURFACE
 #include <cairo-pdf.h>
@@ -150,6 +151,24 @@ public:
 
   SurfaceType get_type() const;
 
+  /**
+   * Emits the current page for backends that support multiple pages,
+   * but doesn't clear it, so that the contents of the current page will
+   * be retained for the next page.  Use show_page() if you want to get an empty
+   * page after the emission.
+   *
+   * @since 1.6
+   */
+  void copy_page();
+
+  /**
+   * Emits and clears the current page for backends that support multiple pages.
+   * Use copy_page() if you don't want to clear the page.
+   *
+   * @since 1.6
+   */
+  void show_page();
+
 #ifdef CAIRO_HAS_PNG_FUNCTIONS
 
   /** Writes the contents of surface to a new file filename as a PNG image.
@@ -276,6 +295,29 @@ public:
    */
   int get_stride() const;
 
+  /**
+   * This function provides a stride value that will respect all
+   * alignment requirements of the accelerated image-rendering code
+   * within cairo. Typical usage will be of the form:
+   *
+   * @code
+   * int stride;
+   * unsigned char *data;
+   * Cairo::RefPtr<Cairo::ImageSurface> surface;
+   *
+   * stride = Cairo::ImageSurface::format_stride_for_width (format, width);
+   * data = malloc (stride * height);
+   * surface = Cairo::ImageSurface::create (data, format, width, height);
+   * @endcode
+   *
+   * @param format A Cairo::Format value
+   * @param width The desired width of an image surface to be created.
+   * @return the appropriate stride to use given the desired format and width, or
+   * -1 if either the format is invalid or the width too large.
+   *
+   * @since 1.6
+   **/
+  static int format_stride_for_width (Cairo::Format format, int width);
 
   /** Creates an image surface of the specified format and dimensions. The
    * initial contents of the surface is undefined; you must explicitely clear
@@ -414,6 +456,15 @@ public:
 
 #ifdef CAIRO_HAS_PS_SURFACE
 
+/**
+ * describes the language level of the PostScript Language Reference that a
+ * generated PostScript file will conform to.
+ */
+typedef enum {
+    PS_LEVEL_2 = CAIRO_PS_LEVEL_2,
+    PS_LEVEL_3 = CAIRO_PS_LEVEL_3
+} PsLevel;
+
 /** A PsSurface provides a way to render PostScript documents from cairo.  This
  * surface is not rendered to the screen but instead renders the drawing to a
  * PostScript file on disk.
@@ -495,6 +546,54 @@ public:
    */
   void dsc_begin_page_setup();
 
+/**
+ * If eps is true, the PostScript surface will output Encapsulated
+ * PostScript.
+ *
+ * This function should only be called before any drawing operations
+ * have been performed on the current page. The simplest way to do
+ * this is to call this function immediately after creating the
+ * surface. An Encapsulated Postscript file should never contain more
+ * than one page.
+ *
+ * @since 1.6
+ **/
+  void set_eps(bool eps);
+
+  /**
+   * Restricts the generated PostSript file to @level. See get_levels() for a
+   * list of available level values that can be used here.
+   *
+   * This function should only be called before any drawing operations have been
+   * performed on the given surface. The simplest way to do this is to call this
+   * function immediately after creating the surface.
+   *
+   * @param level PostScript level
+   *
+   * @since 1.6
+   **/
+  void restrict_to_level(PsLevel level);
+
+  /**
+   * Used to retrieve the list of supported levels. See
+   * restrict_to_level().
+   *
+   * @since 1.6
+   **/
+  static const std::vector<PsLevel> get_levels();
+
+  /**
+   * Get the string representation of the given level id. This function will
+   * return an empty string if level id isn't valid. See get_levels() for a way
+   * to get the list of valid level ids.
+   *
+   * @return the string associated to given level.
+   *
+   * @param level a level id
+   *
+   * @since 1.6
+   **/
+  static std::string level_to_string(PsLevel level);
 };
 
 #endif // CAIRO_HAS_PS_SURFACE
@@ -573,7 +672,7 @@ public:
    * will be empty if version isn't valid. See get_versions() for a way to get
    * the list of valid version ids.
    *
-   * Since: 1.2
+   * since: 1.2
    */
   static std::string version_to_string(SvgVersion version);
 };
